@@ -21,6 +21,7 @@ from .parse import BeautifulSoup
 from .parse import check_page_raise
 from .parse import parse_loggedin_user
 from .parse import parse_page
+from .parse import parse_submission_figures
 from .parse import parse_user_favorites
 from .parse import parse_user_journals
 from .parse import parse_user_submissions
@@ -47,6 +48,7 @@ class FAAPI:
         self.robots: RobotFileParser = get_robots(self.session)  # robots.txt handler
         self.last_get: float = time() - self.crawl_delay  # Time of last get (UNIX time)
         self.raise_for_unauthorized: bool = True  # Control login checks
+        self.timeout: Optional[int] = None  # Timeout for requests
 
     @property
     def user_agent(self) -> str:
@@ -111,7 +113,7 @@ class FAAPI:
         """
         return parse_loggedin_user(self.get_parsed("login", skip_auth_check=True)) is not None
 
-    def get(self, path: str, **params) -> Response:
+    def get(self, path: str, **params: Union[str, bytes, int, float]) -> Response:
         """
         Fetch a path with a GET request.
         The path is checked against the robots.txt before the request is made.
@@ -123,10 +125,10 @@ class FAAPI:
         """
         self.check_path(path, raise_for_disallowed=True)
         self.handle_delay()
-        return get(self.session, path, **params)
+        return get(self.session, path, timeout=self.timeout, params=params)
 
-    def get_parsed(self, path: str, *, skip_page_check: bool = False, skip_auth_check: bool = False, **params
-                   ) -> BeautifulSoup:
+    def get_parsed(self, path: str, *, skip_page_check: bool = False, skip_auth_check: bool = False,
+                   **params: Union[str, bytes, int, float]) -> BeautifulSoup:
         """
         Fetch a path with a GET request and parse it using BeautifulSoup.
 
@@ -153,6 +155,16 @@ class FAAPI:
         """
         return self.user(user) if (user := parse_loggedin_user(self.get_parsed("login"))) else None
 
+    def frontpage(self) -> list[SubmissionPartial]:
+        """
+        Fetch latest submissions from Fur Affinity's front page
+
+        :return: A list of SubmissionPartial objects
+        """
+        page_parsed: BeautifulSoup = self.get_parsed("/")
+        submissions: list[SubmissionPartial] = [SubmissionPartial(f) for f in parse_submission_figures(page_parsed)]
+        return sorted({s.id: s for s in submissions}.values(), reverse=True)
+
     def submission(self, submission_id: int, get_file: bool = False, *, chunk_size: int = None
                    ) -> tuple[Submission, Optional[bytes]]:
         """
@@ -176,7 +188,7 @@ class FAAPI:
         :return: The submission file as a bytes object.
         """
         self.handle_delay()
-        return stream_binary(self.session, submission.file_url, chunk_size=chunk_size)
+        return stream_binary(self.session, submission.file_url, chunk_size=chunk_size, timeout=self.timeout)
 
     def journal(self, journal_id: int) -> Journal:
         """
