@@ -1,7 +1,8 @@
 from datetime import datetime
-from re import Pattern
 from re import compile as re_compile
+from re import IGNORECASE
 from re import match
+from re import Pattern
 from re import search
 from re import sub
 from typing import Any
@@ -13,14 +14,14 @@ from bs4.element import Tag
 from dateutil.parser import parse as parse_date
 
 from .connection import root
+from .exceptions import _raise_exception
 from .exceptions import DisabledAccount
-from .exceptions import NoTitle
 from .exceptions import NonePage
 from .exceptions import NotFound
 from .exceptions import NoticeMessage
+from .exceptions import NoTitle
 from .exceptions import ParsingError
 from .exceptions import ServerError
-from .exceptions import _raise_exception
 
 mentions_regexp: Pattern = re_compile(r"^(?:(?:https?://)?(?:www\.)?furaffinity.net)?/user/([^/#]+).*$")
 watchlist_next_regexp: Pattern = re_compile(r"/watchlist/(by|to)/[^/]+/\d+")
@@ -63,6 +64,128 @@ def username_url(username: str) -> str:
     return sub(r"[^a-z\d.~-]", "", username.lower())
 
 
+def inner_html(tag: Tag) -> str:
+    return tag.decode_contents()
+
+
+def clean_html(html: str) -> str:
+    return sub(r" {2,}", " ", sub(r"[\r\n]", "", html)).strip()
+
+
+def html_to_bbcode(html: str, *, special_characters: bool = False) -> str:
+    body: Optional[Tag] = parse_page(f"<html><body>{html}</body></html>").select_one("html > body")
+    if not body:
+        return ""
+
+    for linkusername in body.select("a.linkusername"):
+        linkusername.replaceWith(f"@{linkusername.text.strip()}")
+
+    for iconusername in body.select("a.iconusername,a.usernameicon"):
+        username: str = iconusername.text.strip() or iconusername.attrs.get('href', '').strip('/').split('/')[-1]
+        if icon := iconusername.select_one("img"):
+            username = icon.attrs.get('alt', '').strip() or username
+        iconusername.replaceWith(f":icon{username}:" if iconusername.text.strip() else f":{username}icon:")
+
+    for img in body.select("img"):
+        img.replaceWith(f"[img={img.attrs.get('src', '')}/]")
+
+    for hr in body.select("hr"):
+        hr.replaceWith("-----")
+
+    for smilie in body.select("i.smilie"):
+        smilie_class: list[str] = list(smilie.attrs.get("class", []))
+        smilie_name: str = next(filter(lambda c: c not in ["smilie", ""], smilie_class), "")
+        smilie.replaceWith(f":{smilie_name or 'smilie'}:")
+
+    for span in body.select("span.bbcode[style*=color]"):
+        if m := match(r".*color: ?([^ ;]+).*", span.attrs["style"]):
+            span.replaceWith(f"[color={m[1]}]{html_to_bbcode(inner_html(span))}[/color]")
+        else:
+            span.replaceWith(html_to_bbcode(inner_html(span)))
+
+    for nav_link in body.select("span.parsed_nav_links"):
+        a_tags = nav_link.select("a")
+        a_prev_tag = next(filter(lambda t: t.text.strip().upper() == "<<<\xA0PREV", a_tags), None)
+        a_frst_tag = next(filter(lambda t: t.text.strip().upper() == "FIRST", a_tags), None)
+        a_last_tag = next(filter(lambda t: t.text.strip().upper() == "NEXT\xA0>>>", a_tags), None)
+        a_prev = a_prev_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_prev_tag else ""
+        a_frst = a_frst_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_frst_tag else ""
+        a_last = a_last_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_last_tag else ""
+        nav_link.replaceWith(f"[{a_prev or '-'},{a_frst or '-'},{a_last or '-'}]")
+
+    for a in body.select("a.auto_link_shortened"):
+        a.replaceWith(f"[url={a.attrs.get('href', '')}]{a.attrs.get('href', '')}[/url]")
+
+    for a in body.select("a"):
+        a.replaceWith(f"[url={a.attrs.get('href', '')}]{html_to_bbcode(inner_html(a))}[/url]")
+
+    for yt in body.select("iframe[src*='youtube.com/embed']"):
+        yt.replaceWith(f"[yt]https://youtube.com/embed/{yt.attrs.get('src', '').strip('/').split('/')}[/yt]")
+
+    for quote_name_tag in body.select("span.bbcode.bbcode_quote > span.bbcode_quote_name"):
+        quote_author: str = quote_name_tag.text.strip().removesuffix('wrote:').strip()
+        quote_tag = quote_name_tag.parent
+        if not quote_tag:
+            quote_name_tag.replaceWith(quote_author)
+            continue
+        quote_name_tag.decompose()
+        quote_tag.replaceWith(f"[quote{('=' + quote_author) if quote_author else ''}]"
+                              f"{html_to_bbcode(inner_html(quote_tag))}"
+                              "[/quote]")
+
+    for quote_tag in body.select("span.bbcode.bbcode_quote"):
+        quote_tag.replaceWith(f"[quote]{html_to_bbcode(inner_html(quote_tag))}[/quote]")
+
+    for [selector, bbcode] in (
+            ("i", "i"),
+            ("b", "b"),
+            ("strong", "b"),
+            ("u", "u"),
+            ("s", "s"),
+            ("code.bbcode_left", "left"),
+            ("code.bbcode_center", "center"),
+            ("code.bbcode_right", "right"),
+            ("span.bbcode_spoiler", "spoiler"),
+            ("sub", "sub"),
+            ("sup", "sup"),
+            ("h1", "h1"),
+            ("h2", "h2"),
+            ("h3", "h3"),
+            ("h4", "h4"),
+            ("h5", "h5"),
+            ("h6", "h6"),
+    ):
+        for tag in body.select(selector):
+            tag.replaceWith(f"[{bbcode}]{html_to_bbcode(inner_html(tag))}[/{bbcode}]")
+
+    for tag in body.select(":not(br)"):
+        if not (div_class := tag.attrs.get("class", None)):
+            tag.replaceWith(f"[tag.{tag.name}]{html_to_bbcode(inner_html(tag))}[/tag.{tag.name}]")
+        else:
+            tag.replaceWith(f"[tag.{tag.name}.{' '.join(div_class) if isinstance(div_class, list) else div_class}]"
+                            f"{html_to_bbcode(inner_html(tag))}"
+                            f"[/tag.{tag.name}]")
+
+    html = body.decode_contents()
+
+    if special_characters:
+        for char, substitution in (
+                (r"<br/?>", "\n"),
+                ("©", "(c)"),
+                ("™", "(tm)"),
+                ("®", "(r)"),
+                ("&copy;", "(c)"),
+                ("&reg;", "(tm)"),
+                ("&trade;", "(r)"),
+                ("&lt;", "<"),
+                ("&gt;", ">"),
+                ("&amp;", "&"),
+        ):
+            html = sub(char, substitution, html, flags=IGNORECASE)
+
+    return html.strip()
+
+
 def parse_mentions(tag: Tag) -> list[str]:
     mentions: list[str] = [username_url(m[1]) for a in tag.select("a")
                            if (m := match(mentions_regexp, get_attr(a, "href")))]
@@ -86,13 +209,14 @@ def parse_journal_section(section_tag: Tag) -> dict[str, Any]:
     assert tag_content is not None, _raise_exception(ParsingError("Missing content tag"))
     assert tag_comments is not None, _raise_exception(ParsingError("Missing comments tag"))
 
+    # noinspection DuplicatedCode
     title: str = tag_title.text.strip()
     date: datetime = parse_date(
         get_attr(tag_date, "title").strip()
         if match(r"^[A-Za-z]+ \d+,.*$", get_attr(tag_date, "title"))
         else tag_date.text.strip()
     )
-    content: str = "".join(map(str, tag_content.children))
+    content: str = clean_html(inner_html(tag_content))
     mentions: list[str] = parse_mentions(tag_content)
     comments: int = int(tag_comments.text.strip())
 
@@ -123,15 +247,16 @@ def parse_journal_page(journal_page: BeautifulSoup) -> dict[str, Any]:
     assert tag_comments is not None, _raise_exception(ParsingError("Missing comments tag"))
 
     id_: int = int(tag_id.attrs.get("content", "0").strip("/").split("/")[-1])
+    # noinspection DuplicatedCode
     title: str = tag_title.text.strip()
     date: datetime = parse_date(
         get_attr(tag_date, "title").strip()
         if match(r"^[A-Za-z]+ \d+,.*$", get_attr(tag_date, "title"))
         else tag_date.text.strip()
     )
-    header: str = "".join(map(str, tag_header.children)).strip() if tag_header else ""
-    footer: str = "".join(map(str, tag_footer.children)).strip() if tag_footer else ""
-    content: str = "".join(map(str, tag_content.children)).strip()
+    header: str = inner_html(tag_header) if tag_header else ""
+    footer: str = inner_html(tag_footer) if tag_footer else ""
+    content: str = clean_html(inner_html(tag_content))
     mentions: list[str] = parse_mentions(tag_content)
     comments: int = int(tag_comments.text.strip())
 
@@ -219,6 +344,7 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
     tag_type: Optional[Tag] = sub_page.select_one("div#submission_page[class^='page-content-type']")
     tag_fav: Optional[Tag] = sub_page.select_one("div.fav > a")
     tag_info: Optional[Tag] = sub_page.select_one("section.info.text")
+    tag_user_folders: list[Tag] = sub_page.select("section.folder-list-container > div > a")
 
     assert tag_info is not None, _raise_exception(ParsingError("Missing info tag"))
 
@@ -269,7 +395,7 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
     comment_count: int = int(tag_comment_count.text.strip())
     favorites: int = int(tag_favorites.text.strip())
     type_: str = tag_type["class"][0][18:]
-    description: str = "".join(map(str, tag_description.children)).strip()
+    description: str = clean_html(inner_html(tag_description))
     mentions: list[str] = parse_mentions(tag_description)
     folder: str = m.group(1).lower() if (m := match(r"^/(scraps|gallery)/.*$", get_attr(tag_folder, "href"))) else ""
     file_url: str = "https:" + get_attr(tag_file_url, "href")
@@ -280,6 +406,16 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
         get_attr(tag_next, "href").split("/")[-2]) if tag_next and tag_next.text.lower() == "next" else None
     fav_link: Optional[str] = f"{root}{href}" if (href := get_attr(tag_fav, "href")).startswith("/fav/") else None
     unfav_link: Optional[str] = f"{root}{href}" if (href := get_attr(tag_fav, "href")).startswith("/unfav/") else None
+    user_folders: list[tuple[str, str, str]] = []
+    for a in tag_user_folders:
+        tag_folder_name: Optional[Tag] = a.select_one("span")
+        tag_folder_group: Optional[Tag] = a.select_one("strong")
+        assert tag_folder_name is not None, _raise_exception(ParsingError("Missing folder name tag"))
+        user_folders.append((
+            tag_folder_name.text.strip(),
+            (root + href) if (href := a.attrs.get("href", "")) else "",
+            tag_folder_group.text.strip() if tag_folder_group else ""
+        ))
 
     return {
         "id": id_,
@@ -298,6 +434,7 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
         "description": description,
         "mentions": mentions,
         "folder": folder,
+        "user_folders": user_folders,
         "file_url": file_url,
         "thumbnail_url": thumbnail_url,
         "prev": prev_sub,
@@ -335,7 +472,7 @@ def parse_user_page(user_page: BeautifulSoup) -> dict[str, Any]:
     name: str = u[1:]
     title: str = ttd[0].strip() if len(ttd := tag_title_join_date.text.rsplit("|", 1)) > 1 else ""
     join_date: datetime = parse_date(ttd[-1].strip().split(":", 1)[1])
-    profile: str = "".join(map(str, tag_profile.children)).strip()
+    profile: str = clean_html(inner_html(tag_profile))
     stats: tuple[int, ...] = (
         *map(lambda s: int(s.split(":")[1]), filter(bool, map(str.strip, tag_stats.text.split("\n")))),
         int(m[1]) if (m := search(r"(\d+)", tag_watchlist_to.text)) else 0,
@@ -400,7 +537,7 @@ def parse_comment_tag(tag: Tag) -> dict:
     assert attr_id is not None, _raise_exception(ParsingError("Missing id attribute"))
 
     comment_id: int = int(attr_id.removeprefix("cid:"))
-    comment_text: str = "".join(map(str, tag_body.children)).strip()
+    comment_text: str = clean_html(inner_html(tag_body))
 
     if tag_username is None:
         return {
